@@ -159,6 +159,7 @@ class KatCP(LineReceiver):
         name, d, queue = self.queries[0]
         if name != msg.name:
             d.errback("Wrong request order")
+            return
         self.queries.pop(0) # hopefully it's not large
         d.callback((queue, msg))
 
@@ -209,8 +210,16 @@ class ServerKatCPProtocol(KatCP):
         and build data
         """
         self.transport.registerProducer(self, True)
-        self.send_message(Message.inform("version", *self.VERSION))
-        self.send_message(Message.inform("build-state", *self.BUILD_STATE))
+        self.send_message(Message.inform("version", self.version()))
+        self.send_message(Message.inform("build-state", self.build_state()))
+
+    def build_state(self):
+        """Return a build state string in the form name-major.minor[(a|b|rc)n]"""
+        return "%s-%s.%s%s" % self.BUILD_STATE
+
+    def version(self):
+        """Return a version string of the form type-major.minor."""
+        return "%s-%s.%s" % self.VERSION
 
     def request_help(self, msg):
         """Return help on the available requests.
@@ -297,10 +306,11 @@ class KatCPServer(Factory):
             client.transport.loseConnection()
 
     def stop(self):
-        self.port.stopListening()
+        result = self.port.stopListening()
         if getattr(self, 'production', None):
             # XXX maybe do it somewhere else???
             reactor.stop()
+        return result
 
     def buildProtocol(self, addr):
         protocol = Factory.buildProtocol(self, addr)
