@@ -3,7 +3,8 @@ from katcp.tx.core import DeviceServer, ClientKatCPProtocol, DeviceProtocol
 from twisted.internet.defer import DeferredList
 from twisted.internet import reactor
 from twisted.internet.protocol import ClientFactory
-from katcp import Message, AsyncReply
+from twisted.python import log
+from katcp import Message, AsyncReply, Sensor
 from katcp.kattypes import request, return_reply, Int
 
 import re, time
@@ -17,19 +18,18 @@ def value_only_formatted(func):
     new_func.func_name = func.func_name
     return new_func
 
-class ProxiedSensor(object):
+class ProxiedSensor(Sensor):
     """ A sensor which is a proxy for other sensor on the remote device.
     Returns a deferred on read
     """
     def __init__(self, name, description, units, stype, device, proxy,
                  *formatted_params):
         self.basename = name
-        self.name = device.name + '.' + name
         self.device = device
-        self.description = description
-        self.units = units
-        self.stype = stype
-        self.formatted_params = formatted_params
+        stype = Sensor.parse_type(stype)
+        params = Sensor.parse_params(stype, formatted_params)
+        Sensor.__init__(self, stype, device.name + '.' + name, description,
+                        units, params=params)
 
     def read_formatted(self):
         return self.device.send_request('sensor-value', self.basename)
@@ -60,6 +60,8 @@ class DeviceHandler(ClientKatCPProtocol):
 
     _conn_counter = 0
 
+    SensorClass = ProxiedSensor
+
     def __init__(self, name, host, port):
         self.name = name
         self.host = host
@@ -69,29 +71,36 @@ class DeviceHandler(ClientKatCPProtocol):
         self.sensors = {}
         self.state = self.UNSYNCED
 
+    def _got_help(self, (informs, reply)):
+        for inform in informs:
+            self.requests.append(inform.arguments[0])
+        self.send_request('sensor-list').addCallback(self._got_sensor_list)
+
+    def _got_sensor_list(self, (informs, reply)):
+        self.state = self.SYNCED
+        for inform in informs:
+            name, description, units, stype = inform.arguments[:4]
+            formatted_arguments = inform.arguments[4:]
+            sensor = self.SensorClass(name, description,
+                                      units, stype,
+                                      self, self.proxy, *formatted_arguments)
+            self.sensors[name] = sensor
+            self.proxy.add_proxied_sensor(self, sensor)
+        self.device_ready()
+        self.proxy.device_ready(self)
+
+    def device_ready(self):
+        """ Another hook that can be overloaded if you want to execute
+        code just after device has been synced
+        """
+        pass
+
     def connectionMade(self):
         """ This is called after connection has been made. Introspect server
         about it's capabilities
         """
-        def got_help((informs, reply)):
-            for inform in informs:
-                self.requests.append(inform.arguments[0])
-            self.send_request('sensor-list').addCallback(got_sensor_list)
-
-        def got_sensor_list((informs, reply)):
-            self.state = self.SYNCED
-            for inform in informs:
-                name, description, units, stype = inform.arguments[:4]
-                formatted_arguments = inform.arguments[4:]
-                sensor = ProxiedSensor(name, description,
-                                       units, stype,
-                                       self, self.proxy, *formatted_arguments)
-                self.sensors[name] = sensor
-                self.proxy.add_proxied_sensor(self, sensor)
-            self.proxy.device_ready(self)
-
         self.state = self.SYNCING
-        self.send_request('help').addCallback(got_help)
+        self.send_request('help').addCallback(self._got_help)
         self._conn_counter = 0
 
     def add_proxy(self, proxy):
@@ -114,6 +123,14 @@ class DeviceHandler(ClientKatCPProtocol):
         """
         pass
 
+    def inform_sensor_status(self, msg):
+        try:
+            sensor = self.sensors[msg.arguments[2]]
+            sensor.set_formatted(msg.arguments[0], msg.arguments[3],
+                                 msg.arguments[4])
+        except:
+            log.err()
+
 class ProxyProtocol(DeviceProtocol):
     @request(include_msg=True)
     @return_reply(Int(min=0))
