@@ -3,7 +3,7 @@ from katcp.tx.core import DeviceServer, ClientKatCPProtocol
 from katcp.tx.proxy import ProxyKatCP, DeviceHandler, DeviceProtocol
 from twisted.trial.unittest import TestCase
 from twisted.internet.protocol import ClientCreator
-from twisted.internet.defer import Deferred
+from twisted.internet.defer import Deferred, inlineCallbacks
 from katcp import Sensor, Message
 from katcp.kattypes import request, return_reply, Int
 from twisted.internet import reactor
@@ -31,6 +31,12 @@ class ExampleDevice(DeviceServer):
         sensor2._timestamp = 0
         self.add_sensor(sensor2)
 
+class MyDeviceHandler(DeviceHandler):
+    ready = False
+    
+    def device_ready(self):
+        self.ready = True
+
 class ExampleProxy(ProxyKatCP):
     on_device_ready = None
     CONN_DELAY_TIMEOUT = 0.05
@@ -41,12 +47,12 @@ class ExampleProxy(ProxyKatCP):
         self.finish = finish
 
     def setup_devices(self):
-        dev2 = DeviceHandler('device2', 'localhost', 6)
+        dev2 = MyDeviceHandler('device2', 'localhost', 6)
         dev2.connectionMade = lambda *args: None
         dev2._conn_counter = 100
         self.add_device(dev2)
         self.ready_devices = 1
-        self.add_device(DeviceHandler('device', 'localhost', self.connect_to))
+        self.add_device(MyDeviceHandler('device', 'localhost', self.connect_to))
 
     def devices_scan_complete(self):
         self.finish.callback(None)
@@ -177,6 +183,24 @@ class TestProxyBase(TestCase):
 
         return self._base_test(('sensor-list', '/state/'), callback)
 
+    def test_sensor_sampling(self):
+        def check_value():
+            assert self.proxy.sensors['device.sensor1'].value() == 10
+            self.port.stopListening()
+            self.proxy.stop()
+            self.finish.callback(None)
+
+        def sampling_done((informs, reply)):
+            self.example_device.sensors['sensor1'].set_value(10)
+            reactor.callLater(0.1, check_value)
+        
+        def callback(arg):
+            d = self.proxy.devices['device'].send_request(
+                'sensor-sampling', 'sensor1', 'period', '10')
+            d.addCallback(sampling_done)
+            return True
+        return self._base_test(None, callback)
+
     def test_reconnect_base(self):
         def works((informs, reply)):
             self.assertEquals(reply, Message.reply('device-watchdog', 'ok'))
@@ -208,6 +232,13 @@ class TestProxyBase(TestCase):
 
         return self._base_test(('watchdog',), callback)
 
+    def test_device_ready(self):
+        def callback(_):
+            device = self.proxy.devices['device']
+            assert device.ready == True
+        
+        return self._base_test(None, callback)
+
     def test_halt(self):
         def callback((informs, reply)):
             self.assertEquals(reply, Message.reply('halt', 'device', 'ok'))
@@ -220,7 +251,7 @@ class RogueSensor(object):
     description = 'descr'
     units = 'some'
     stype = 'integer'
-    formatted_params = ()
+    formatted_params = (0, 10)
 
     def __init__(self, name, device):
         self.device = device
